<?php

namespace Drupal\gnetwork\Plugin\Field\FieldFormatter;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\gnetwork\Plugin\Field\FieldType\G6SyncItem;

/**
 * Plugin implementation of the 'gnet_g6sync_default' formatter.
 *
 * @FieldFormatter(
 *   id = "gnet_g6sync_default",
 *   label = @Translation("Default"),
 *   field_types = {"gnet_g6sync"}
 * )
 */
class G6SyncDefaultFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return ['foo' => 'bar'] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();
    $element['foo'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Foo'),
      '#default_value' => $settings['foo'],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $settings = $this->getSettings();
    $summary[] = $this->t('Foo: @foo', ['@foo' => $settings['foo']]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {

      if ($item->uid_created) {
        $allowed_values = G6SyncItem::allowedUserCreatedValues();
        $element[$delta]['uid_created'] = [
          '#type' => 'item',
          '#title' => $this->t('User created'),
          '#markup' => $allowed_values[$item->uid_created],
        ];
      }

      if ($item->uid_changed) {
        $allowed_values = G6SyncItem::allowedUserChangedValues();
        $element[$delta]['uid_changed'] = [
          '#type' => 'item',
          '#title' => $this->t('User changed'),
          '#markup' => $allowed_values[$item->uid_changed],
        ];
      }

      if ($item->ts_created) {
        $allowed_values = G6SyncItem::allowedCreatedOnValues();
        $date = DrupalDateTime::createFromFormat('Y-m-d\TH:i:s', $item->ts_created);
        // @DCG: Consider injecting the date formatter service.
        // @codingStandardsIgnoreStart
        $date_formatter = \Drupal::service('date.formatter');
        // @codingStandardsIgnoreStart
        $timestamp = $date->getTimestamp();
        $formatted_date = $allowed_values[$item->ts_created];
        $iso_date = $date_formatter->format($timestamp, 'custom', 'Y-m-d\TH:i:s') . 'Z';
        $element[$delta]['ts_created'] = [
          '#type' => 'item',
          '#title' => $this->t('Created on'),
          'content' => [
            '#theme' => 'time',
            '#text' => $formatted_date,
            '#html' => FALSE,
            '#attributes' => [
              'datetime' => $iso_date,
            ],
            '#cache' => [
              'contexts' => [
                'timezone',
              ],
            ],
          ],
        ];
      }

      if ($item->ts_changed) {
        $allowed_values = G6SyncItem::allowedChangedOnValues();
        $date = DrupalDateTime::createFromFormat('Y-m-d\TH:i:s', $item->ts_changed);
        // @DCG: Consider injecting the date formatter service.
        // @codingStandardsIgnoreStart
        $date_formatter = \Drupal::service('date.formatter');
        // @codingStandardsIgnoreStart
        $timestamp = $date->getTimestamp();
        $formatted_date = $allowed_values[$item->ts_changed];
        $iso_date = $date_formatter->format($timestamp, 'custom', 'Y-m-d\TH:i:s') . 'Z';
        $element[$delta]['ts_changed'] = [
          '#type' => 'item',
          '#title' => $this->t('Changed on'),
          'content' => [
            '#theme' => 'time',
            '#text' => $formatted_date,
            '#html' => FALSE,
            '#attributes' => [
              'datetime' => $iso_date,
            ],
            '#cache' => [
              'contexts' => [
                'timezone',
              ],
            ],
          ],
        ];
      }

    }

    return $element;
  }

}
