<?php

namespace Drupal\gnetwork\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Defines the 'gnet_g6sync' field type.
 *
 * @FieldType(
 *   id = "gnet_g6sync",
 *   label = @Translation("g6 Sync"),
 *   category = @Translation("General"),
 *   default_widget = "gnet_g6sync",
 *   default_formatter = "gnet_g6sync_default"
 * )
 */
class G6SyncItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    $settings = ['foo' => 'example'];
    return $settings + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $settings = $this->getSettings();

    $element['foo'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Foo'),
      '#default_value' => $settings['foo'],
      '#disabled' => $has_data,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    $settings = ['bar' => 'example'];
    return $settings + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();

    $element['bar'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Foo'),
      '#default_value' => $settings['bar'],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    if ($this->uid_created !== NULL) {
      return FALSE;
    }
    elseif ($this->uid_changed !== NULL) {
      return FALSE;
    }
    elseif ($this->ts_created !== NULL) {
      return FALSE;
    }
    elseif ($this->ts_changed !== NULL) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {

    $properties['uid_created'] = DataDefinition::create('integer')
      ->setLabel(t('User created'));
    $properties['uid_changed'] = DataDefinition::create('integer')
      ->setLabel(t('User changed'));
    $properties['ts_created'] = DataDefinition::create('datetime_iso8601')
      ->setLabel(t('Created on'));
    $properties['ts_changed'] = DataDefinition::create('datetime_iso8601')
      ->setLabel(t('Changed on'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraints = parent::getConstraints();

    $options['uid_created']['AllowedValues'] = array_keys(G6SyncItem::allowedUserCreatedValues());

    $options['uid_changed']['AllowedValues'] = array_keys(G6SyncItem::allowedUserChangedValues());

    $options['ts_created']['AllowedValues'] = array_keys(G6SyncItem::allowedCreatedOnValues());

    $options['ts_changed']['AllowedValues'] = array_keys(G6SyncItem::allowedChangedOnValues());

    $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();
    $constraints[] = $constraint_manager->create('ComplexData', $options);
    // @todo Add more constraints here.
    return $constraints;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {

    $columns = [
      'uid_created' => [
        'type' => 'int',
        'size' => 'normal',
      ],
      'uid_changed' => [
        'type' => 'int',
        'size' => 'normal',
      ],
      'ts_created' => [
        'type' => 'varchar',
        'length' => 20,
      ],
      'ts_changed' => [
        'type' => 'varchar',
        'length' => 20,
      ],
    ];

    $schema = [
      'columns' => $columns,
      // @DCG Add indexes here if necessary.
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {

    $values['uid_created'] = array_rand(self::allowedUserCreatedValues());

    $values['uid_changed'] = array_rand(self::allowedUserChangedValues());

    $values['ts_created'] = array_rand(self::allowedCreatedOnValues());

    $values['ts_changed'] = array_rand(self::allowedChangedOnValues());

    return $values;
  }

  /**
   * Returns allowed values for 'uid_created' sub-field.
   *
   * @return array
   *   The list of allowed values.
   */
  public static function allowedUserCreatedValues() {
    return [
      123 => 123,
      456 => 456,
      789 => 789,
    ];
  }

  /**
   * Returns allowed values for 'uid_changed' sub-field.
   *
   * @return array
   *   The list of allowed values.
   */
  public static function allowedUserChangedValues() {
    return [
      123 => 123,
      456 => 456,
      789 => 789,
    ];
  }

  /**
   * Returns allowed values for 'ts_created' sub-field.
   *
   * @return array
   *   The list of allowed values.
   */
  public static function allowedCreatedOnValues() {
    return [
      '2018-01-01T00:10:10' => '1 January 2018, 00:10:10',
      '2018-02-01T00:20:20' => '1 February 2018, 00:20:20',
      '2018-03-01T00:30:30' => '1 March 2018, 00:30:30',
    ];
  }

  /**
   * Returns allowed values for 'ts_changed' sub-field.
   *
   * @return array
   *   The list of allowed values.
   */
  public static function allowedChangedOnValues() {
    return [
      '2018-01-01T00:10:10' => '1 January 2018, 00:10:10',
      '2018-02-01T00:20:20' => '1 February 2018, 00:20:20',
      '2018-03-01T00:30:30' => '1 March 2018, 00:30:30',
    ];
  }

}
